! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: ebkgmap_skyframe
!
! Authors:  Masaaki Sakano - SSC/LUX
!
! Description: Shared routines for ebkgmap package to get the Sky coordinate frame information.
! 
! Comments: This file is aimed to be imported to ssclib or attcalc2 at some point.
! 
! ToDo lists: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module ebkgmap_skyframe

  use dal
  use cal

  ! Followings in ssclib
  use ssc_misc
  use cal_aux
  use dal_aux
  use wcs_aux		! in ssclib

  implicit none 

  private 

  public getSkyCoordFrame

contains 

  !---------------------------------------------------------------------
  subroutine getSkyCoordFrame(skyTemplateImageSetName&
     , wcs, xiSize, yiSize&
     , expStartTimeStamp, medianScAttitude, instrumentId, instrumAttitude)
  !---------------------------------------------------------------------

    implicit none 

    !-- Description --
    ! get Sky coordinates data etc from the template image.
    !! nb., this routine comes from ./eootemap_mod.f90

    character(*), intent(in) :: skyTemplateImageSetName
    type(WcsT), intent(out) :: wcs	! type def in ssclib/wcs_aux
    integer, intent(out), optional :: xiSize, yiSize
    real(double), intent(out), optional :: expStartTimeStamp
    type(SpacecraftAttitudeType), intent(out), optional :: medianScAttitude
    integer(int32), intent(out), optional :: instrumentId
    type(SpacecraftAttitudeType), intent(out), optional :: instrumAttitude
      ! scAttitude, instrumAttitude, medianScAttitude, dummyInstrumAttitude

    type(DataSetT) :: imgSet
    type(ArrayT) :: imArray
    real(single), pointer :: templateImage(:,:)
    integer(int32) :: instrumentIdLocal
    real(double) :: expStartTimeStampLocal
    type(SpacecraftAttitudeType) :: medianScAttitudeLocal
    type(SpacecraftAttitudeType) :: instrumAttitudeLocal

    !-------------------------------------------------------------------
    ! Get sky coordinates.

    imgSet = dataSet(skyTemplateImageSetName, READ)
    imArray = array(imgSet, 0)
    call readWcs(imgSet, imArray, wcs) ! in ssclib/wcs_aux.

    if (     (present(instrumentId))     .or. (present(expStartTimeStamp))&
        .or. (present(medianScAttitude)) .or. (present(instrumAttitude))) then
      call getCalInfo(imgSet, instrumentIdLocal&
         , expStartTimeStampLocal, medianScAttitudeLocal, instrumAttitudeLocal)
      ! in ssclib/cal_aux.
      if (present(instrumentId)) instrumentId = instrumentIdLocal
      if (present(expStartTimeStamp)) expStartTimeStamp = expStartTimeStampLocal
      if (present(medianScAttitude)) medianScAttitude = medianScAttitudeLocal
      if (present(instrumAttitude)) instrumAttitude = instrumAttitudeLocal
    end if

    if ((present(xiSize)) .or. (present(yiSize))) then
      call readArrayData(imArray, templateImage) ! in ssclib/dal_aux.
      if (present(xiSize)) xiSize = size(templateImage, 1)
      if (present(yiSize)) yiSize = size(templateImage, 2)
      deallocate(templateImage)
    end if

    call release(imgSet)

  end subroutine getSkyCoordFrame

end module ebkgmap_skyframe
