! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: epfluormap_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: This task makes a map of one of the PN fluorescence lines.
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module epfluormap_mod

  use dal
  use param  
  use errorHandling
  use cal

  use cube_aux
  use eimchip2sky_aux

  use dal_aux
  use cal_aux
  use wcs_aux
!use dump

  implicit none 

  private 

  public epfluormap

  character(255) ::&
    expMapCubeSetName='',&
    templateImageSetName='',&
    fluorMapCubeSetName='',&
    fluorMapSetName='',&
    binnedAttSetName = '',&
    tempSetName=''
  character(20) ::&
    attitudeStyle = ''

contains 

  !---------------------------------------------------------------------
  subroutine epfluormap
  !---------------------------------------------------------------------

    implicit none 

    real(single), pointer :: expMapCube(:,:,:), fluorMapCube(:,:,:)
    integer(int32) ::&
      expmapInstrId
    type(SpacecraftAttitudeType) ::&
      scAttitude, instrumAttitude
    real(double) ::&
      expStartTimeStamp
    integer(int16) ::&
      chipXsize, chipYsize
    integer(int8) ::&
      numCcds, numNodes, ccdNum, nodeNum, ccdNodeNum, maxCcdNodeNum
    integer(int8), allocatable :: ccd2ccdnr(:,:)
    type(intervalVecType), allocatable :: gtiVec(:)
    type(WcsT) :: wcs

    !-------------------------------------------------------------------
    ! Start:

    call readParameters

    call readArrayData(expMapCubeSetName, expMapCube) ! in ssclib/dal_aux.
    call getCalInfo(expMapCubeSetName, expmapInstrId, expStartTimeStamp&
      , scAttitude, instrumAttitude) ! in ssclib/cal_aux.

if (expmapInstrId .ne. EPN) call error('',"")

    chipXSize = EPN_MAX_X	! All these are defined in caloaldefs.
    chipYSize = EPN_MAX_Y
    numCcds = EPN_CCD_LAST
    numNodes = 1

    allocate(ccd2ccdnr(numCcds, numNodes))
    do ccdNum = 1, numCcds
      do nodeNum = 1, numNodes
        ccd2ccdnr(ccdNum, nodeNum) = 10 * (nodeNum - 1) + ccdNum
      end do
    end do

    maxCcdNodeNum = 10 * (numNodes - 1) + numCcds

if (maxCcdNodeNum .ne. size(expMapCube, 1)) call error('',"")
if (chipXSize     .ne. size(expMapCube, 2)) call error('',"")
if (chipYSize     .ne. size(expMapCube, 3)) call error('',"")

    call readArrayData(fluorMapCubeSetName, fluorMapCube) ! in ssclib/dal_aux.

if (maxCcdNodeNum .ne. size(fluorMapCube, 1)) call error('',"")
if (chipXSize     .ne. size(fluorMapCube, 2)) call error('',"")
if (chipYSize     .ne. size(fluorMapCube, 3)) call error('',"")

    fluorMapCube = fluorMapCube * expMapCube

    deallocate(expMapCube)

    call readWcs(templateImageSetName, wcs) ! in ssclib/wcs_aux.

    call writeCubeInSkyCoords(fluorMapCube&
      , reshape(ccd2ccdnr, (/maxCcdNodeNum/)), expStartTimeStamp, wcs&
      , gtiVec, templateImageSetName, templateImageSetName, fluorMapSetName&
      , tempSetName, 'counts', .false.&
      , attitudeStyle, binnedAttSetName)
    ! in eimchip2sky_aux.

    deallocate(fluorMapCube)

    ! Deallocate all the gtis:

    do ccdNum = 1, numCcds
      do nodeNum = 1, numNodes
        ccdNodeNum = ccd2ccdnr(ccdNum, nodeNum)
        deallocate(gtiVec(ccdNodeNum)%gtis)
      end do
    end do

  end subroutine epfluormap

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none 

    !-------------------------------------------------------------------
    ! Start:

    expMapCubeSetName   = stringParameter('expcubeset')
    fluorMapCubeSetName = stringParameter('fluorcubeset')
    fluorMapSetName     = stringParameter('fluormapset')
    templateImageSetName = stringParameter('templateset')
    attitudeStyle = stringParameter('attstyle')

    select case(attitudeStyle)
    case('binnedset')
      binnedAttSetName = stringParameter('binnedattset')
    case('template')
call error('notYetSupported',"") !** just for the present.
    case default
call error('',"")
    end select

!    tempSetName          = stringParameter('tempset')
tempSetName = 'tempset.ds'

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none 

    !-------------------------------------------------------------------
    ! Start:

  end subroutine template

end module epfluormap_mod


