package TestEbkgmap;
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
# Originally taken from TestLccorr.pm .

our $VERSION = '1.0.0';

use strict;
use Data::Dumper;
use SSCLib;
use Fitsplutils;

=head1 NAME

TestEbkgmap - Common subroutines for testing ebkgmap.

=head1 SYNOPSIS

    use TestEbkgmap

=head1 DESCRIPTION

Every subroutine is called as

  my $ret = &TestEbkgmap::THIS_SUBROUTINE(...);

=item prepare_data ( )

=item getisrun ( $runbin, Default )

Returns the array of run(1) or not(0).  The first index is always "".
The input is the binary of "11011" etc, where the highest is for the first.

=cut
  ;	# For Emacs


############################################################

## Global variables
# our $Defaultdir = "outfiles";

#---------------------------------------------------------------------
sub getisrun {
#---------------------------------------------------------------------
  ## Input: ($runbin, Default)	# Binary of "11011" etc, where the highest is for the first.
  ## Return: (@isrun)	# Array of run(1) or not(0)
  ## Description: Return the array which shows 1 or 0 for each
  ##   index.  The first one ($isrun[0]) is always "".

  my($runstr) = shift; 
  my($runstrdef) = shift; 
  my(@isrun);

  $runstrdef = "11111111111111111111111111111" if ((! defined($runstrdef)) || ($runstrdef eq ""));
  $runstr = $runstrdef if ((! defined($runstr)) || ($runstr eq ""));
  my($a) = length($runstrdef)-length($runstr);
  if ($a > 0) {
    $runstr .= substr($runstrdef, length($runstr), $a);	# Fill with Defaults where not specified 
  }

  @isrun = split('', $runstr);
  unshift @isrun, "";

  return @isrun;	# ("", 1, 0, 1, 1, 1) etc.

  ## Unit-Test: &TestLccorr::getisrun("11", "10111") => ( '', '1', '1', '1', '1', '1' )
  ## Unit-Test: &TestLccorr::getisrun("11", "10101") => ( '', '1', '1', '1', '0', '1' )
  ## Unit-Test: &TestLccorr::getisrun("001","10111") => ( '', '0', '0', '1', '1', '1' )
}


1;
