#!/usr/local/bin/perl
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

package prepdata;

require 5.6.0;	# to support "0b" binary expression and our()

use SSCLib;		# in ssclib
use File::Basename;

sub prepare_data {
    ## Input: An observation identifier of the form '0455_0111360101_002', and path to unzipped location
    ## Return: 0 if normal ends.
    ## Description: mainly gunzip the data, if needed.
    ## Reference: /lccorr/test/TestLccorr.pm
    my $obsid = shift;
    my $indatadir = shift;

    my ($i);
    my (@files);
    my (@InFitsFile);
    my ($stat) = 0;
    my $data_dir = &SSCLib::gettestdata_dir('ebkgmap', $obsid);

    @InFitsFile = glob("$data_dir/*.FTZ");
    @files = map {$_=basename($_);s/\.FTZ$/.FIT/;$_=$indatadir."/".$_} @InFitsFile; # All the gzipped FITS files

    foreach my $file (@files) {
        $stat += abs( &SSCLib::gunzipfits_if_needed($file, $data_dir) );

        # Select a short interval in time if we have an event list
        if ( $file =~ /(.+IEVLI.+)\..+/ ) {
            my $basename = $1;

            my $shortname = "${basename}_SHORT.FIT";

            if ( ! -e $shortname ) {

                my $firsttime = `fdump ${file}[EVENTS] outfile=STDOUT columns=TIME rows=1 prhead=no showcol=no showunit=no showrow=no`;
                my $lasttime = $firsttime + 5000;   # Use max this number of seconds of data

                %hsinopt
                    = ( 'table'                => $file
                        , 'withfilteredset'    => "true"
                        , 'destruct'           => "yes"
                        , 'keepfilteroutput' => "true"
                        , 'expression'         => "TIME.lt.$lasttime"
                        , 'filteredset'        => $shortname
                        , '-V 6'               => undef );

                my $status = &SSCLib::run( &SSCLib::getsascommandstring("evselect", \%hsinopt), (! $main::Verbose));
                if ($status) {
                    print "prepdata.pm: Failed to execute evselect\n";
                    exit(1);
                }

                # Overwrite the original with the shortened selection
                `cp $shortname ${basename}.FIT`;
            }

        }
    }

    return($stat);
}

1;
