! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testeootepileupmaskaux
  use errorhandling
  ! use types
  use dal		! 
  use cal

  use eootepileupmask_aux
  use epileupmask_shared
  ! use eimchip2sky_aux

  !! Followings are in ssclib
  use array_utils	! for smallestTrueIndex
  use coordinate	! for rawToDet
  !use constants		! for Pi
  use test_utils	! for isNearlyEqual()

  implicit none

  call test_calcRgaSpikeMaskProjM1
  call test_calcRgaSpikeMaskProjM2

contains

  !---------------------------------------------------------------------
  subroutine test_calcRgaSpikeMaskProjM1
  !---------------------------------------------------------------------
  ! Tests of calcRgaSpikeMaskProjected() for EMOS1 (M1).
  ! Note: 

    implicit none

    character(32)  :: errHead
    type(infoCcdChipsT) :: infoCcdChips	! Defined in epileupmask_shared
    logical, pointer :: maskCube(:,:,:) => null()
    ! integer(int16) :: chipXSize, chipYSize
    integer(int32), parameter :: instrumentId = EMOS1	! in caloalutils/src/caloaldefs.f90
    integer, pointer :: cubePlaneNums(:) => null()
    logical, pointer :: maskCubeXOnly(:,:) => null(), maskCubeYOnly(:,:) => null()
    integer(int16) :: i, ci, xi, yi, xiChip(7), yiChip(7)
    ! integer :: k
    real(double) :: xDetFrom, yDetFrom, xDetTo, yDetTo, xDetChip(7), yDetChip(7)
    ! integer, parameter :: precision = 5

    !-------------------------------------------------------------------
    ! Start:

    call CAL_setState(instrument=instrumentId)
    call CAL_setState(randomize=.false.)

    infoCcdChips = getDefaultInfoCcdChipsT(instrumentId)

    allocate(maskCube(infoCcdChips%numCcds, infoCcdChips%chipXSize, infoCcdChips%chipYSize) &
      , cubePlaneNums(infoCcdChips%numCcds))
    maskCube = .false.
    xiChip(:) = -99
    yiChip(:) = -99
    xDetChip = 0.0d0
    yDetChip = 0.0d0

    cubePlaneNums = (/ (i, i=1,infoCcdChips%numCcds) /)	! (1, 2, ..., 7) for MOSs
    ! See comments in  countNumberOfFrames() in epileupmask_shared.f90 for the definitions of cubePlaneNums(:)

    !----------------------------------------------------------
    ! calcRgaSpikeMaskProjected()
    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1001a"
    print *,"=== "//trim(errHead)//": Basic: Chip1(300,300)=T"

    ci = 1
    xi = 300
    yi = 300
    maskCube(ci,xi,yi) = .true.
    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId & !, numCubePlanes
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 3, 6 has a T for each.

    if      (any(maskCubeXOnly(1:2,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1:2,:) should be F.')
    else if (any(maskCubeXOnly(4:5,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(4:5,:) should be F.')
    else if (any(maskCubeXOnly(7,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(7,:) should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(3,:)) = ', count(maskCubeXOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(6,:)) = ', count(maskCubeXOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeYOnly(ci, yi)) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1,300) should be T.')
    end if
    
    ! Check the T in Chip 3 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeXOnly(3,:))	! defined in ssclib/array_utils
    xiChip(3) = i
    call CAL_setState(ccdChipId=int(3, int32))
    call rawToDet(i, 100_int16, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    yDetChip(3) = yDetTo 
    ! if (.not. isNearlyEqual(yDetFrom, yDetTo, 3)) then	! too close to zero.
    if (abs(yDetFrom - yDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 3: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 6 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeXOnly(6,:))	! defined in ssclib/array_utils
    xiChip(6) = i
    call CAL_setState(ccdChipId=int(6, int32))
    call rawToDet(i, 100_int16, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    yDetChip(6) = yDetTo 
    if (abs(yDetFrom - yDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 6: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      ! -- Output --
      ! index in Chip 6: x =  297
      ! (xDetFrom, yDetFrom)= -10.9715065387709103 -10.9715065387709103
      ! (xDetTo,   yDetTo)  =  -1.7590554606078571E+04 -21.6708293477495673
      call error("err"//trim(errHead), '')
    end if

    maskCubeYOnly(ci, yi) = .false.	! Force to change to check as follows.
    if (any(maskCubeYOnly)) then
      call error("err"//trim(errHead), 'maskCubeYOnly(else) should be F.')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1002a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 3 to 1, 6"

    ci = 3
    xi = xiChip(3)
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    ! maskCubeXOnly = .false.
    ! maskCubeYOnly = .false.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId & !, numCubePlanes
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 3, 6 has a T for each.

    if      (any(maskCubeXOnly(1:2,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1:2,:) should be F.')
    else if (any(maskCubeXOnly(4:5,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(4:5,:) should be F.')
    else if (any(maskCubeXOnly(7,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(7,:) should be F.')
    else if (any(maskCubeYOnly(2:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(2:7,:) should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 3 and 6
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly) .ne. 1) then	! in Chip 1
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(3,:)) = ', count(maskCubeXOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(6,:)) = ', count(maskCubeXOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(1,:)) = ', count(maskCubeYOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 3 (original)
    i = smallestTrueIndex(maskCubeXOnly(3,:))	! defined in ssclib/array_utils
    if (xiChip(3) .ne. i) then
      print *,"index in Chip 3: x = ",i
      print *,"(xiChip(3))=",xiChip(3)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 3
    i = smallestTrueIndex(maskCubeYOnly(1,:))	! defined in ssclib/array_utils
    if (yiChip(1) .ne. i) then
      print *,"index in Chip 1: y = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 6 is in line with that in Chip 3
    i = smallestTrueIndex(maskCubeXOnly(6,:))	! defined in ssclib/array_utils
    if (abs(xiChip(6) - i) > 1) then
      print *,"index in Chip 6: x = ",i	! =296
      print *,"(xiChip(6))=",xiChip(6)	! =297
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetChip(6), yDetChip(6))=",xDetChip(6),yDetChip(6)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1003a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 6 to 1, 3"

    ci = 6
    xi = xiChip(6)
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 3, 6 has a T for each.

    if      (any(maskCubeXOnly(1:2,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1:2,:) should be F.')
    else if (any(maskCubeXOnly(4:5,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(4:5,:) should be F.')
    else if (any(maskCubeXOnly(7,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(7,:) should be F.')
    else if (any(maskCubeYOnly(2:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(2:7,:) should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 3 and 6
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly) .ne. 1) then	! in Chip 1
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(3,:)) = ', count(maskCubeXOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(6,:)) = ', count(maskCubeXOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(1,:)) = ', count(maskCubeYOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 6 (original)
    i = smallestTrueIndex(maskCubeXOnly(ci,:))	! defined in ssclib/array_utils
    if (xiChip(ci) .ne. i) then
      print *,"index in Chip 6: x = ",i
      print *,"(xiChip(6))=",xiChip(6)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 6
    i = smallestTrueIndex(maskCubeYOnly(1,:))	! defined in ssclib/array_utils
    if (yiChip(1) .ne. i) then
      print *,"index in Chip 1: y = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetChip(6), yDetChip(6))=",xDetChip(6),yDetChip(6)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 3 is in line with that in Chip 6
    i = smallestTrueIndex(maskCubeXOnly(3,:))	! defined in ssclib/array_utils
    if (abs(xiChip(3) - i) > 1) then
      print *,"index in Chip 3: x = ",i	! =
      print *,"(xiChip(3))=",xiChip(3)	! =
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetChip(6), yDetChip(6))=",xDetChip(6),yDetChip(6)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1012a"
    print *,"=== "//trim(errHead)//": Basic: Chip4(300,300)=T"

    ci = 4
    xi = 300
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 4, 5 has a T for each.

    if      (any(maskCubeXOnly(1:3,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1:3,:) should be F.')
    else if (any(maskCubeXOnly(6:7,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(6:7,:) should be F.')
    else if (any(maskCubeYOnly)) then
      call error("err"//trim(errHead), 'maskCubeYOnly should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 4 and 5
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(4,:)) = ', count(maskCubeXOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(5,:)) = ', count(maskCubeXOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 4 (original)
    i = smallestTrueIndex(maskCubeXOnly(4,:))	! defined in ssclib/array_utils
    if (xiChip(4) .ne. i) then
      print *,"index in Chip 4: x = ",i
      print *,"(xiChip(4))=",xiChip(4)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 5 is in line with that in Chip 4
    i = smallestTrueIndex(maskCubeXOnly(5,:))	! defined in ssclib/array_utils
    xiChip(5) = i
    call CAL_setState(ccdChipId=int(5, int32))
    call rawToDet(i, 100_int16, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    yDetChip(5) = yDetTo 
    if (abs(yDetFrom - yDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 5: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1013a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 5 to 4"

    ci = 5
    xi = xiChip(5)
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 4, 5 has a T for each.

    if      (any(maskCubeXOnly(1:3,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1:3,:) should be F.')
    else if (any(maskCubeXOnly(6:7,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(6:7,:) should be F.')
    else if (any(maskCubeYOnly)) then
      call error("err"//trim(errHead), 'maskCubeYOnly should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 4 and 5
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(4,:)) = ', count(maskCubeXOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(5,:)) = ', count(maskCubeXOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 5 (original)
    i = smallestTrueIndex(maskCubeXOnly(ci,:))	! defined in ssclib/array_utils
    if (xiChip(ci) .ne. i) then
      print *,"index in Chip 5: x = ",i
      print *,"(xiChip(5))=",xiChip(5)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 is in line with that in Chip 5
    i = smallestTrueIndex(maskCubeXOnly(4,:))	! defined in ssclib/array_utils
    if (abs(xiChip(4) - i) > 1) then
      print *,"index in Chip 4: x = ",i	! =
      print *,"(xiChip(4))=",xiChip(4)	! =
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1022a"
    print *,"=== "//trim(errHead)//": Basic: Chip2(300,300)=T"

    ci = 2
    xi = 300
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 2, 7 has a T for each.

    if      (any(maskCubeXOnly(1,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1,:) should be F.')
    else if (any(maskCubeXOnly(3:6,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(3:6,:) should be F.')
    else if (any(maskCubeYOnly)) then
      call error("err"//trim(errHead), 'maskCubeYOnly should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 2 and 7
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(2,:)) = ', count(maskCubeXOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(7,:)) = ', count(maskCubeXOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 2 (original)
    i = smallestTrueIndex(maskCubeXOnly(2,:))	! defined in ssclib/array_utils
    if (xiChip(2) .ne. i) then
      print *,"index in Chip 2: x = ",i
      print *,"(xiChip(2))=",xiChip(2)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 is in line with that in Chip 2
    i = smallestTrueIndex(maskCubeXOnly(7,:))	! defined in ssclib/array_utils
    xiChip(7) = i
    call CAL_setState(ccdChipId=int(7, int32))
    call rawToDet(i, 100_int16, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    yDetChip(7) = yDetTo 
    if (abs(yDetFrom - yDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 7: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM1023a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 7 to 2"

    ci = 7
    xi = xiChip(7)
    yi = 300
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 2, 7 has a T for each.

    if      (any(maskCubeXOnly(1,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1,:) should be F.')
    else if (any(maskCubeXOnly(3:6,:))) then
      call error("err"//trim(errHead), 'maskCubeXOnly(3:6,:) should be F.')
    else if (any(maskCubeYOnly)) then
      call error("err"//trim(errHead), 'maskCubeYOnly should be F.')
    else if (count(maskCubeXOnly) .ne. 2) then	! in Chip 2 and 7
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      do i=1,7
         print *, i, count(maskCubeXOnly(i,:))
      end do
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(2,:)) = ', count(maskCubeXOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(7,:)) = ', count(maskCubeXOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      print *, 'maskCubeXOnly(',ci,',',xi,') = ', maskCubeXOnly(ci, xi)
      call error("err"//trim(errHead), '')
    end if
    
    ! Check the T in Chip 7 (original)
    i = smallestTrueIndex(maskCubeXOnly(ci,:))	! defined in ssclib/array_utils
    if (xiChip(ci) .ne. i) then
      print *,"index in Chip 7: x = ",i
      print *,"(xiChip(7))=",xiChip(7)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 2 is in line with that in Chip 7
    i = smallestTrueIndex(maskCubeXOnly(2,:))	! defined in ssclib/array_utils
    if (abs(xiChip(2) - i) > 1) then
      print *,"index in Chip 2: x = ",i	! =
      print *,"(xiChip(2))=",xiChip(2)	! =
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! Cleaning up

    deallocate(maskCubeXOnly, maskCubeYOnly)
    deallocate(maskCube, cubePlaneNums)

! !     !----------------------------------------------------------
! !     errHead = "AngleBetweenCelCoords002a"
! !     print *,"=== "//errHead//": Radian (Pi)(0), Single"
! ! 
! !     vecAS = real(vecAD, single)
! !     vecBS = real(vecBD, single)
! !     angleS = angleBetweenCelCoords(vecAS, vecBS)
! !     if (.not. isNearlyEqual(angleS, Pi, precision)) then
! !       print *,"angle=",angleS
! !       call error("err"//errHead, '')
! !     end if
! 

  end subroutine test_calcRgaSpikeMaskProjM1


  !---------------------------------------------------------------------
  subroutine test_calcRgaSpikeMaskProjM2
  !---------------------------------------------------------------------
  ! Tests of calcRgaSpikeMaskProjected() for EMOS2 (M2).
  ! Note: 

    implicit none

    character(32)  :: errHead
    type(infoCcdChipsT) :: infoCcdChips	! Defined in epileupmask_shared
    logical, pointer :: maskCube(:,:,:) => null()
    ! integer(int16) :: chipXSize, chipYSize
    integer(int32), parameter :: instrumentId = EMOS2	! in caloalutils/src/caloaldefs.f90
    integer, pointer :: cubePlaneNums(:) => null()
    logical, pointer :: maskCubeXOnly(:,:) => null(), maskCubeYOnly(:,:) => null()
    integer(int16) :: i, ci, xi, yi, xiChip(7), yiChip(7)
    ! integer :: k
    real(double) :: xDetFrom, yDetFrom, xDetTo, yDetTo, xDetChip(7), yDetChip(7)
    ! integer, parameter :: precision = 5

    !-------------------------------------------------------------------
    ! Start:

    call CAL_setState(instrument=instrumentId)
    call CAL_setState(randomize=.false.)

    infoCcdChips = getDefaultInfoCcdChipsT(instrumentId)

    allocate(maskCube(infoCcdChips%numCcds, infoCcdChips%chipXSize, infoCcdChips%chipYSize) &
      , cubePlaneNums(infoCcdChips%numCcds))
    maskCube = .false.
    xiChip(:) = -99
    yiChip(:) = -99
    xDetChip = 0.0d0
    yDetChip = 0.0d0

    cubePlaneNums = (/ (i, i=1,infoCcdChips%numCcds) /)	! (1, 2, ..., 7) for MOSs
    ! See comments in  countNumberOfFrames() in epileupmask_shared.f90 for the definitions of cubePlaneNums(:)

    !----------------------------------------------------------
    ! calcRgaSpikeMaskProjected()
    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2001a"
    print *,"=== "//trim(errHead)//": Basic: Chip1(99,99)=T (to Chip5,7)"

    ci = 1
    xi = 99
    yi = 99
    ! maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    ! maskCube(ci,xi,yi) = .true.

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 5, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      do i=1,7
        if (any(maskCubeYOnly(i,:))) print *,'any(Y) is yes: ',i, ' count=', count(maskCubeYOnly(i,:))&
           , ' smallest=',smallestTrueIndex(maskCubeYOnly(i,:))
        if (any(maskCubeXOnly(i,:))) print *,'any(X) is yes: ',i, ' count=', count(maskCubeXOnly(i,:))
      end do
      print *,maskCubeYOnly(3,:)
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (any(maskCubeYOnly(6,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(6,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1,99) should be T.')
    end if

    ! Check the T in Chip 5 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    yiChip(5) = i
    call CAL_setState(ccdChipId=int(5, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(5) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 5: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(7,:))	! defined in ssclib/array_utils
    yiChip(7) = i
    call CAL_setState(ccdChipId=int(7, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(7) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 7: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    maskCubeXOnly(ci, xi) = .false.	! Force to change to check as follows.
    if (any(maskCubeXOnly)) then
      call error("err"//trim(errHead), 'maskCubeXOnly(else) should be F.')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2002a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 5 to 1, 7"

    ci = 5
    xi = 99
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 5, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (any(maskCubeYOnly(6,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(6,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeYOnly(ci, yi)) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5,99) should be T.')
    end if

    ! Check the T in Chip 5 (original)
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    if (yiChip(5) .ne. i) then
      print *,"index in Chip 5: y = ",i
      print *,"(yiChip(5))=",yiChip(5)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 5
    i = smallestTrueIndex(maskCubeXOnly(1,:))	! defined in ssclib/array_utils
    if (xiChip(1) .ne. i) then
      print *,"index in Chip 1: x = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 is in line with that in Chip 5
    i = smallestTrueIndex(maskCubeYOnly(7,:))	! defined in ssclib/array_utils
    if (abs(yiChip(7) - i) > 1) then
      print *,"index in Chip 7: y = ",i	! =
      print *,"(xiChip(7))=",yiChip(7)	! =
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2003a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 7 to 1, 5"

    ci = 7
    xi = 99
    yi = yiChip(7)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 5, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (any(maskCubeYOnly(6,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(6,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeYOnly(ci, yi)) then
      call error("err"//trim(errHead), 'maskCubeYOnly(7,99) should be T.')
    end if

    ! Check the T in Chip 7 (original)
    i = smallestTrueIndex(maskCubeYOnly(ci,:))	! defined in ssclib/array_utils
    if (yiChip(ci) .ne. i) then
      print *,"index in Chip 7: y = ",i
      print *,"(xiChip(7))=",yiChip(7)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 7
    i = smallestTrueIndex(maskCubeXOnly(1,:))	! defined in ssclib/array_utils
    if (xiChip(1) .ne. i) then
      print *,"index in Chip 1: x = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 5 is in line with that in Chip 7
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    if (abs(yiChip(5) - i) > 1) then
      print *,"index in Chip 5: y = ",i	! =
      print *,"(xiChip(5))=",yiChip(5)	! =
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2011a"
    print *,"=== "//trim(errHead)//": Basic: Chip1(499,99)=T (to Chip2,4)"

    ci = 1
    xi = 499
    yi = 99
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 5, 7 has a T for each.

    if      (any(maskCubeYOnly(3,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(3,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeXOnly(ci, xi)) then
      call error("err"//trim(errHead), 'maskCubeXOnly(1,499) should be T.')
    end if

    ! Check the T in Chip 2 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    yiChip(2) = i
    call CAL_setState(ccdChipId=int(2, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(2) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 2: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(4,:))	! defined in ssclib/array_utils
    yiChip(4) = i
    call CAL_setState(ccdChipId=int(4, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(4) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 4: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    maskCubeXOnly(ci, xi) = .false.	! Force to change to check as follows.
    if (any(maskCubeXOnly)) then
      call error("err"//trim(errHead), 'maskCubeXOnly(else) should be F.')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2012a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 2 to 1, 4"

    ci = 2
    xi = 499
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 2, 4 has a T for each.

    if      (any(maskCubeYOnly(3,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(3,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeYOnly(ci, yi)) then
      write(errStr, '("maskCubeYOnly(",i1,",",i3," should be T.")') ci, yi
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 2 (original)
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    if (yiChip(2) .ne. i) then
      print *,"index in Chip 2: y = ",i
      print *,"(yiChip(2))=",yiChip(2)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 2
    i = smallestTrueIndex(maskCubeXOnly(1,:))	! defined in ssclib/array_utils
    if (xiChip(1) .ne. i) then
      print *,"index in Chip 1: x = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 is in line with that in Chip 2
    i = smallestTrueIndex(maskCubeYOnly(4,:))	! defined in ssclib/array_utils
    if (abs(yiChip(4) - i) > 1) then
      print *,"index in Chip 4: y = ",i	! =
      print *,"(xiChip(4))=",yiChip(4)	! =
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2013a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 4 to 1, 2"

    ci = 4
    xi = 499
    yi = yiChip(4)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 1, 2, 4 has a T for each.

    if      (any(maskCubeYOnly(3,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(3,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 2) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeXOnly(1,:)) .ne. 1) then
      print *, 'count(maskCubeXOnly(1,:)) = ', count(maskCubeXOnly(1,:))
      call error("err"//trim(errHead), '')
    else if (.not. maskCubeYOnly(ci, yi)) then
      write(errStr, '("maskCubeYOnly(",i1,",",i3," should be T.")') ci, yi
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 (original)
    i = smallestTrueIndex(maskCubeYOnly(ci,:))	! defined in ssclib/array_utils
    if (yiChip(ci) .ne. i) then
      print *,"index in Chip 4: y = ",i
      print *,"(xiChip(4))=",yiChip(4)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 1 is in line with that in Chip 4
    i = smallestTrueIndex(maskCubeXOnly(1,:))	! defined in ssclib/array_utils
    if (xiChip(1) .ne. i) then
      print *,"index in Chip 1: x = ",i
      print *,"(xiChip(1), yiChip(1))=",xiChip(1),yiChip(1)
      print *,"(xDetChip(1), yDetChip(1))=",xDetChip(1),yDetChip(1)
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 2 is in line with that in Chip 4
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    if (abs(yiChip(2) - i) > 1) then
      print *,"index in Chip 2: y = ",i	! =
      print *,"(xiChip(2))=",yiChip(2)	! =
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2021a"
    print *,"=== "//trim(errHead)//": Basic: Chip6(99,499)=T (to Chip5,7)"

    ci = 6
    xi = 99
    yi = 499
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 5, 6, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(6,:)) = ', count(maskCubeYOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 6 (original)
    if (.not. maskCubeYOnly(6,yiChip(6))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(6,yiChip(6)) = F')
    end if

    ! Check the T in Chip 5 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    yiChip(5) = i
    call CAL_setState(ccdChipId=int(5, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(5) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 5: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(7,:))	! defined in ssclib/array_utils
    yiChip(7) = i
    call CAL_setState(ccdChipId=int(7, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(7) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 7: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2022a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 5 to 6, 7"

    ci = 5
    xi = 99
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 5, 6, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(6,:)) = ', count(maskCubeYOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 5 (original)
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    if (yiChip(5) .ne. i) then
      print *,"index in Chip 5: y = ",i
      print *,"(yiChip(5))=",yiChip(5)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 6 is in line with that in Chip 5
    i = smallestTrueIndex(maskCubeYOnly(6,:))	! defined in ssclib/array_utils
    if (abs(yiChip(6) - i) > 1) then
      print *,"index in Chip 6: y = ",i	! =
      print *,"(xiChip(6))=",yiChip(6)	! =
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetChip(6), yDetChip(6))=",xDetChip(6),yDetChip(6)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 is in line with that in Chip 5
    i = smallestTrueIndex(maskCubeYOnly(7,:))	! defined in ssclib/array_utils
    if (abs(yiChip(7) - i) > 1) then
      print *,"index in Chip 7: y = ",i	! =
      print *,"(xiChip(7))=",yiChip(7)	! =
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2023a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 7 to 5, 6"

    ci = 7
    xi = 99
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 5, 6, 7 has a T for each.

    if      (any(maskCubeYOnly(1:4,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:4,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(5,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(5,:)) = ', count(maskCubeYOnly(5,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(6,:)) = ', count(maskCubeYOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(7,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(7,:)) = ', count(maskCubeYOnly(7,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 7 (original)
    i = smallestTrueIndex(maskCubeYOnly(7,:))	! defined in ssclib/array_utils
    if (yiChip(7) .ne. i) then
      print *,"index in Chip 7: y = ",i
      print *,"(yiChip(7))=",yiChip(7)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 6 is in line with that in Chip 7
    i = smallestTrueIndex(maskCubeYOnly(6,:))	! defined in ssclib/array_utils
    if (abs(yiChip(6) - i) > 1) then
      print *,"index in Chip 6: y = ",i	! =
      print *,"(xiChip(6))=",yiChip(6)	! =
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetChip(6), yDetChip(6))=",xDetChip(6),yDetChip(6)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 5 is in line with that in Chip 7
    i = smallestTrueIndex(maskCubeYOnly(5,:))	! defined in ssclib/array_utils
    if (abs(yiChip(5) - i) > 1) then
      print *,"index in Chip 5: y = ",i	! =
      print *,"(xiChip(5))=",yiChip(5)	! =
      print *,"(xDetChip(7), yDetChip(7))=",xDetChip(7),yDetChip(7)
      print *,"(xDetChip(5), yDetChip(5))=",xDetChip(5),yDetChip(5)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2031a"
    print *,"=== "//trim(errHead)//": Basic: Chip3(99,499)=T (to Chip2,4)"

    ci = 3
    xi = 99
    yi = 499
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call CAL_setState(ccdChipId=int(ci, int32))
    call rawToDet(xi, yi, xDetFrom, yDetFrom)	! defined in ssclib/coordinate
    xiChip(ci) = xi
    yiChip(ci) = yi
    xDetChip(ci) = xDetFrom
    yDetChip(ci) = yDetFrom

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 2, 3, 4 has a T for each.

    if      (any(maskCubeYOnly(1,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(3,:)) = ', count(maskCubeYOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 3 (original)
    if (.not. maskCubeYOnly(3,yiChip(3))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(3,yiChip(3)) = F')
    end if

    ! Check the T in Chip 2 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    yiChip(2) = i
    call CAL_setState(ccdChipId=int(2, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(2) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 2: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 is in line with that in Chip 1
    i = smallestTrueIndex(maskCubeYOnly(4,:))	! defined in ssclib/array_utils
    yiChip(4) = i
    call CAL_setState(ccdChipId=int(4, int32))
    call rawToDet(100_int16, i, xDetTo, yDetTo)	! 100 is arbitrary; defined in ssclib/coordinate
    xDetChip(4) = xDetTo 
    if (abs(xDetFrom - xDetTo)/80 > 0.2) then	! 80 is the physical binning unit to make a pixel size 4 arcsec - hence 0.2 corresponds to 0.8 arcsec.
      print *,"index in Chip 4: x = ",i
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2032a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 2 to 3, 4"

    ci = 2
    xi = 99
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 2, 3, 4 has a T for each.

    if      (any(maskCubeYOnly(1,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(3,:)) = ', count(maskCubeYOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 2 (original)
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    if (yiChip(2) .ne. i) then
      print *,"index in Chip 2: y = ",i
      print *,"(yiChip(2))=",yiChip(2)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 3 is in line with that in Chip 2
    i = smallestTrueIndex(maskCubeYOnly(3,:))	! defined in ssclib/array_utils
    if (abs(yiChip(3) - i) > 1) then
      print *,"index in Chip 3: y = ",i	! =
      print *,"(xiChip(3))=",yiChip(3)	! =
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 is in line with that in Chip 2
    i = smallestTrueIndex(maskCubeYOnly(4,:))	! defined in ssclib/array_utils
    if (abs(yiChip(4) - i) > 1) then
      print *,"index in Chip 4: y = ",i	! =
      print *,"(xiChip(4))=",yiChip(4)	! =
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2033a"
    print *,"=== "//trim(errHead)//": Reverse from Chip 4 to 2, 3"

    ci = 4
    xi = 99
    yi = yiChip(ci)
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 2, 3, 4 has a T for each.

    if      (any(maskCubeYOnly(1,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1,:) should be F.')
    else if (any(maskCubeYOnly(5:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(5:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 3) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(2,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(2,:)) = ', count(maskCubeYOnly(2,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(3,:)) = ', count(maskCubeYOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(4,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(4,:)) = ', count(maskCubeYOnly(4,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 4 (original)
    i = smallestTrueIndex(maskCubeYOnly(4,:))	! defined in ssclib/array_utils
    if (yiChip(4) .ne. i) then
      print *,"index in Chip 4: y = ",i
      print *,"(yiChip(4))=",yiChip(4)
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 3 is in line with that in Chip 4
    i = smallestTrueIndex(maskCubeYOnly(3,:))	! defined in ssclib/array_utils
    if (abs(yiChip(3) - i) > 1) then
      print *,"index in Chip 3: y = ",i	! =
      print *,"(xiChip(3))=",yiChip(3)	! =
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetChip(3), yDetChip(3))=",xDetChip(3),yDetChip(3)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if

    ! Check the T in Chip 2 is in line with that in Chip 4
    i = smallestTrueIndex(maskCubeYOnly(2,:))	! defined in ssclib/array_utils
    if (abs(yiChip(2) - i) > 1) then
      print *,"index in Chip 2: y = ",i	! =
      print *,"(xiChip(2))=",yiChip(2)	! =
      print *,"(xDetChip(4), yDetChip(4))=",xDetChip(4),yDetChip(4)
      print *,"(xDetChip(2), yDetChip(2))=",xDetChip(2),yDetChip(2)
      print *,"(xDetFrom, yDetFrom)=",xDetFrom,yDetFrom
      print *,"(xDetTo,   yDetTo)  =",xDetTo,yDetTo
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2041a"
    print *,"=== "//trim(errHead)//": Basic: Chip6(99,99)=T (to no Chip)"

    ci = 6
    xi = 99
    yi = 99
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 6 has a T for each.

    if      (any(maskCubeYOnly(1:5,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:5,:) should be F.')
    else if (any(maskCubeYOnly(7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 1) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(6,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(6,:)) = ', count(maskCubeYOnly(6,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "CalcRgaSpikeMaskProjectedM2051a"
    print *,"=== "//trim(errHead)//": Basic: Chip3(99,99)=T (to no Chip)"

    ci = 3
    xi = 99
    yi = 99
    maskCube           = .false.
    maskCube(ci,xi,yi) = .true.
    deallocate(maskCubeXOnly, maskCubeYOnly)

    call calcRgaSpikeMaskProjected(maskCube &
     , instrumentId &
     , cubePlaneNums&
     , maskCubeXOnly, maskCubeYOnly)
    ! => Chip 3 has a T for each.

    if      (any(maskCubeYOnly(1:2,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(1:2,:) should be F.')
    else if (any(maskCubeYOnly(4:7,:))) then
      call error("err"//trim(errHead), 'maskCubeYOnly(4:7,:) should be F.')
    else if (count(maskCubeYOnly) .ne. 1) then
      print *, 'count(maskCubeYOnly) = ', count(maskCubeYOnly)
      call error("err"//trim(errHead), '')
    else if (count(maskCubeYOnly(3,:)) .ne. 1) then
      print *, 'count(maskCubeYOnly(3,:)) = ', count(maskCubeYOnly(3,:))
      call error("err"//trim(errHead), '')
    else if (any(maskCubeXOnly)) then
      print *, 'count(maskCubeXOnly) = ', count(maskCubeXOnly)
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! Cleaning up

    deallocate(maskCubeXOnly, maskCubeYOnly)
    deallocate(maskCube, cubePlaneNums)

! !     !----------------------------------------------------------
! !     errHead = "AngleBetweenCelCoords002a"
! !     print *,"=== "//errHead//": Radian (Pi)(0), Single"
! ! 
! !     vecAS = real(vecAD, single)
! !     vecBS = real(vecBD, single)
! !     angleS = angleBetweenCelCoords(vecAS, vecBS)
! !     if (.not. isNearlyEqual(angleS, Pi, precision)) then
! !       print *,"angle=",angleS
! !       call error("err"//errHead, '')
! !     end if
! 

  end subroutine test_calcRgaSpikeMaskProjM2

end program testeootepileupmaskaux
