! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testepileupmaskshared
  use errorhandling
  ! use types
  use dal		! For INTEGER_NULL
  use cal

  use epileupmask_shared
  ! use eimchip2sky_aux

  !! Followings are in ssclib
  !use constants		! for Pi
  use test_utils	! for isNearlyEqual()

  implicit none

  ! call test_rawImageToDet

contains

  !---------------------------------------------------------------------
  subroutine test_rawImageToDet
  !---------------------------------------------------------------------
!   ! Tests of rawImageToDet().
!   ! Note: 
! 
!     implicit none
! 
!     character(25)  :: errHead
!     integer(int16) :: chipXSize, chipYSize
!     integer(int8)  :: numCcds !, numNodes
!     integer(int32), parameter :: instrumentId = EMOS1	! in caloalutils/src/caloaldefs.f90
!     real(single), allocatable :: chipImage(:,:), outImage(:,:)
!     integer(int8)  :: ccdNum
! 
!     type(DataSetT) :: set
!     type(ArrayT) :: aryImageT
!     real(single), dimension(:,:), pointer :: aryImagePtr
!     integer(int32), pointer :: newDimVector(:)
! 
!     integer, parameter :: precision = 5
! 
!     !-------------------------------------------------------------------
!     ! Start:
! 
!     call CAL_setState(instrument=instrumentId)
! 
!     if (instrumentId == EPN) then
!       chipXSize = EPN_MAX_X	! All these are defined in caloaldefs.
!       chipYSize = EPN_MAX_Y
!       numCcds = EPN_CCD_LAST
!       ! numNodes = 1
!       ! tlminLocal = 0
!       ! tlmaxLocal = 32767	! Default, probably used if the input is Image.
! 
!     else if (instrumentId == EMOS1 .or. instrumentId == EMOS2) then
!       chipXSize = EMOS_MAX_X
!       chipYSize = EMOS_MAX_Y
!       numCcds = EMOS_CCD_LAST
!       ! numNodes = 1
!       ! ! numNodes = 2
!       ! ! In principle MOSs can have 2 nodes.  However, in practice,
!       ! ! two nodes have never been used in the actual observations
!       ! ! except for a few calibration observations.
!       ! tlminLocal = 0
!       ! tlmaxLocal = 15000	! Default, probably used if the input is Image.
! 
!     else
! call error('badInstrument',"")
!     end if
! 
!     allocate(chipImage(chipXSize, chipYSize))
!     allocate(outImage(648,648))	! ???
! 
!     !----------------------------------------------------------
!     ! rawImageToDet()
!     !----------------------------------------------------------
!     errHead = "RawImageToDet001a"
!     print *,"=== "//trim(errHead)//": Basic: image=0.0"
! 
!     ccdNum=1	! 1 <= ccdNum <= 7 for MOSs
!     call CAL_setState(ccdChipId=int(ccdNum, int32))
!     if (instrumentId == EMOS1 .or. instrumentId == EMOS2) call CAL_setState(ccdNodeId=0_int32)
! 
!     chipImage = 0.0
!     outImage = 0.0
!     call rawImageToDet(chipImage, ccdNum &
!        , outImage, .false., 'rate')
!        ! , outImage, doApprox=.false., averagingStyle='rate')
! 
!     if (any(outImage .ne. 0)) then
!       call error("err"//trim(errHead), '')
!     end if
! 
!     !----------------------------------------------------------
!     errHead = "RawImageToDet002a"
!     print *,"=== "//trim(errHead)//": Basic: image=1.0"
! 
!     chipImage = 1.0
!     call rawImageToDet(chipImage, ccdNum &
!        , outImage, .false., 'rate')
!        ! , outImage, doApprox=.false., averagingStyle='rate')
! 
!     ! Output to a file "outimage.ds"
!     set = dataSet("outfiles/outimage.ds", CREATE)
! 
!     allocate(newDimVector(0:1))
!     newDimVector = (/size(outImage,1), size(outImage,2)/)
!     aryImageT = addArray(set, 'PRIMARY', REAL32, newDimVector)
!     deallocate(newDimVector)
! 
!     aryImagePtr => real32Array2Data(aryImageT)
!     aryImagePtr = outImage
! 
!     call release(set)
! 
! 
!     if (all(outImage == 0)) then
!       call error("err"//trim(errHead), 'outImage is zero.')
!     else if (any(outImage < 0)) then
!       call error("err"//trim(errHead), 'outImage has a negative value(s).')
!     else if (abs(abs(real(chipXSize)*chipYSize/sum(outImage)) - 1.0) > 0.2) then
!       print *, 'chipXSize=',chipXSize, ' chipYSize=',chipYSize
!       print *, 'sum(Original) =',real(chipXSize)*chipYSize
!       print *, 'sum(chipImage)=',sum(chipImage)	! 3.6000000E+05
!       print *, 'sum(Output)   =',sum(outImage)	! 4.1990400E+05
!       call error("err"//trim(errHead), '')
!     end if
! 
! 
!     ! vecAD = (/Pi,    0.0d0/)	! Pi defined in coordinates
!     ! vecBD = (/0.0d0, 0.0d0/)
!     ! angleD = angleBetweenCelCoords(vecAD, vecBD)
!     ! if (.not. isNearlyEqual(angleD, Pi, precision)) then
!     !   print *,"angle=",angleD
!     !   call error("err"//errHead, '')
!     ! end if
! 
! !     !----------------------------------------------------------
! !     errHead = "AngleBetweenCelCoords002a"
! !     print *,"=== "//errHead//": Radian (Pi)(0), Single"
! ! 
! !     vecAS = real(vecAD, single)
! !     vecBS = real(vecBD, single)
! !     angleS = angleBetweenCelCoords(vecAS, vecBS)
! !     if (.not. isNearlyEqual(angleS, Pi, precision)) then
! !       print *,"angle=",angleS
! !       call error("err"//errHead, '')
! !     end if
! 
  end subroutine test_rawImageToDet


end program testepileupmaskshared
